<?php

use Fpdf\Fpdf;
use PhpOffice\PhpPresentation\PhpPresentation;
use PhpOffice\PhpPresentation\Shape\RichText;
use PhpOffice\PhpPresentation\Shape\Drawing;


error_reporting(E_ALL);
ini_set('display_errors', 0); // Disable error display
ini_set('log_errors', 1);    // Enable error logging
ini_set('error_log', 'error.log'); // Log file path

require 'vendor/autoload.php'; // Include Composer autoloader

header('Content-Type: application/json'); // Ensure JSON output
ob_start();

function curl_get_contents($url)
{
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
    $data = curl_exec($ch);
    curl_close($ch);
    return $data;
}

// Function for parallel download using cURL multi
function multi_curl_get_contents($urls)
{
    $mh = curl_multi_init();
    $handles = [];
    $responses = [];

    foreach ($urls as $i => $url) {
        $handles[$i] = curl_init($url);
        curl_setopt($handles[$i], CURLOPT_RETURNTRANSFER, true);
        curl_setopt($handles[$i], CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($handles[$i], CURLOPT_TIMEOUT, 30);
        curl_multi_add_handle($mh, $handles[$i]);
    }

    $active = null;
    do {
        $mrc = curl_multi_exec($mh, $active);
    } while ($mrc == CURLM_CALL_MULTI_PERFORM);

    while ($active && $mrc == CURLM_OK) {
        if (curl_multi_select($mh) != -1) {
            do {
                $mrc = curl_multi_exec($mh, $active);
            } while ($mrc == CURLM_CALL_MULTI_PERFORM);
        }
    }

    foreach ($handles as $i => $handle) {
        $responses[$i] = curl_multi_getcontent($handle);
        curl_multi_remove_handle($mh, $handle);
    }

    curl_multi_close($mh);
    return $responses;
}
$directoryPath = '/home/handsyeo/public_html/downloads/';
try {
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        $rawInput = file_get_contents('php://input');
        $input = json_decode($rawInput, true);

        if (!isset($input['url']) || !filter_var($input['url'], FILTER_VALIDATE_URL)) {
            echo json_encode(['success' => false, 'message' => 'Invalid URL provided.']);
            exit;
        }

        $slideshareUrl = $input['url'];
        $slidesName = preg_replace('/[^a-zA-Z0-9\-]/', '', basename(parse_url($slideshareUrl, PHP_URL_PATH))) ?: 'slideshare';
        if ($input['type'] === 'ppt') {
            $pptPath = $directoryPath . $slidesName . '.pptx';
            if (file_exists($pptPath)) {
                echo json_encode(['success' => true, 'message' => 'PPT File already exists.', 'filename' => $slidesName . '.pptx']);
                exit;
            }
        }
        if ($input['type'] === 'pdf') {
            $pdfPath = $directoryPath . $slidesName . '.pdf';
            if (file_exists($pdfPath)) {
                echo json_encode(['success' => true, 'message' => 'PDF File already exists.', 'filename' => $slidesName . '.pdf']);
                exit;
            }
        }
        if ($input['type'] === 'zip') {
            $zipFilePath = $directoryPath . $slidesName . '.zip';
            if (file_exists($zipFilePath)) {
                echo json_encode(['success' => true, 'message' => 'ZIP archive already exists.', 'filename' => $slidesName . '.zip']);
                exit;
            }
        }
        // Fetch SlideShare HTML using cURL
        $html = curl_get_contents($slideshareUrl);
        if ($html === false) {
            echo json_encode(['success' => false, 'message' => 'Failed to fetch the SlideShare page.']);
            exit;
        }

        // Extract image URLs using regex
        preg_match_all('/srcSet="([^"]+)"/', $html, $matches);
        if (empty($matches[1])) {
            echo json_encode(['success' => false, 'message' => 'No images found on the SlideShare page.']);
            exit;
        }

        $imageUrls = [];
        foreach ($matches[1] as $srcSet) {
            foreach (explode(',', $srcSet) as $url) {
                $urlParts = explode(' ', trim($url));
                $imageUrl = $urlParts[0];
                if (
                    strpos($imageUrl, 'image.slidesharecdn.com') !== false &&
                    preg_match('/-638\.jpg$/', $imageUrl)
                ) {
                    $imageUrls[] = $imageUrl;
                }
            }
        }

        $uniqueUrls = array_unique($imageUrls);

        if ($input['type'] === 'pdf') {
            $pdf = new FPDF();
            $tempFiles = [];
            $imageContents = multi_curl_get_contents($uniqueUrls);
        
            // Helper function to generate alternate URLs based on known suffixes.
            // It checks if the original URL contains any one of the known suffixes
            // (here, -396.jpg, -320.jpg, or -987.jpg) and returns an array of alternate URLs.
            function getAlternateUrls($originalUrl) {
                $alternates = [];
                // Define the list of possible suffixes.
                $suffixes = ['-396.jpg', '-320.jpg', '-987.jpg'];
                foreach ($suffixes as $suffix) {
                    if (strpos($originalUrl, $suffix) !== false) {
                        // For each suffix other than the one found, create an alternate URL.
                        foreach ($suffixes as $altSuffix) {
                            if ($altSuffix !== $suffix) {
                                $alternates[] = str_replace($suffix, $altSuffix, $originalUrl);
                            }
                        }
                        break; // We only replace one matching suffix.
                    }
                }
                return $alternates;
            }
        
            foreach ($imageContents as $index => $imageContent) {
                // Get the original URL for logging and fallback purposes.
                $originalUrl = $uniqueUrls[$index];
                $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
                file_put_contents($tempImagePath, $imageContent);
        
                $imageInfo = @getimagesize($tempImagePath);
                // If the downloaded file isn't a valid image, try alternate quality URLs.
                if ($imageInfo === false) {
                    unlink($tempImagePath); // Delete the invalid file.
                    $alternates = getAlternateUrls($originalUrl);
                    $success = false;
        
                    foreach ($alternates as $altUrl) {
                        // Attempt to fetch the alternate image.
                        $altImageContent = @file_get_contents($altUrl);
                        if ($altImageContent !== false) {
                            // Write the alternate image to a new temporary file.
                            $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
                            file_put_contents($tempImagePath, $altImageContent);
                            $imageInfo = @getimagesize($tempImagePath);
                            if ($imageInfo !== false) {
                                $success = true;
                                $originalUrl = $altUrl;
                                break;
                            } else {
                                unlink($tempImagePath);
                            }
                        }
                    }
                    if (!$success) {
                        $errorLog = "Error: Invalid Slide Detected after fallback attempts\n";
                        $errorLog .= "Time: " . date("Y-m-d H:i:s") . "\n";
                        $errorLog .= "Original URL: $originalUrl\n";
                        if (!empty($alternates)) {
                            $errorLog .= "Fallback URLs attempted: " . implode(', ', $alternates) . "\n";
                        }
                        $errorLog .= "--------------------------------------\n";
                        file_put_contents('file_error.txt', $errorLog, FILE_APPEND);
                        echo json_encode([
                            'success' => false,
                            'message' => 'Downloaded file is not a valid Slide. Try to download in different quality.'
                        ]);
                        exit;
                    }
                }
        
                // Dynamically set the page size based on the image dimensions (convert pixels to mm)
                $width = $imageInfo[0] * 0.264583;
                $height = $imageInfo[1] * 0.264583;
        
                $pdf->AddPage($width > $height ? 'L' : 'P', [$width, $height]);
        
                // Center the image on the page
                $x = ($pdf->GetPageWidth() - $width) / 2;
                $y = ($pdf->GetPageHeight() - $height) / 2;
        
                $pdf->Image($tempImagePath, $x, $y, $width, $height);
                $tempFiles[] = $tempImagePath;
            }
        
            // Save the PDF to file.
            $pdf->Output($pdfPath, 'F');
        
            // Cleanup temporary images.
            foreach ($tempFiles as $tempFile) {
                if (file_exists($tempFile)) {
                    unlink($tempFile);
                }
            }
        
            echo json_encode([
                'success' => true,
                'message' => 'PDF generated successfully.',
                'filename' => $slidesName . '.pdf'
            ]);
            exit;
        } elseif ($input['type'] === 'zip') {
            // Generate ZIP archive
            $zip = new ZipArchive();

            if ($zip->open($zipFilePath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === true) {
                // Download all image data in parallel
                $imageContents = multi_curl_get_contents($uniqueUrls);

                foreach ($imageContents as $index => $imageContent) {
                    if ($imageContent !== false) {
                        $fileName = basename($uniqueUrls[$index]);
                        $zip->addFromString($fileName, $imageContent);

                        // Apply compression for this file
                        $zip->setCompressionIndex($index, ZipArchive::CM_DEFLATE);
                    }
                }
                $zip->close();
            } else {
                echo json_encode(['success' => false, 'message' => 'Failed to create ZIP archive.']);
                exit;
            }
            echo json_encode(['success' => true, 'message' => 'ZIP archive generated successfully.', 'filename' => $slidesName . '.zip']);
            exit;
        } elseif ($input['type'] === 'ppt') {
            $presentation = new PhpPresentation();
            $presentation->removeSlideByIndex(0);
            $tempFiles = []; // Track temp files for cleanup
            $imageContents = multi_curl_get_contents($uniqueUrls);
        
            // foreach ($imageContents as $index => $imageContent) {
            // $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
            // file_put_contents($tempImagePath, $imageContent);
            // $tempFiles[] = $tempImagePath;
    
            // // Get image dimensions
            // $imageInfo = getimagesize($tempImagePath);
            // if ($imageInfo === false) {
            //     continue; // Skip invalid images
            // }
            
            foreach ($imageContents as $index => $imageContent) {
                if (@imagecreatefromstring($imageContent) === false) {
                    continue; // Skip invalid image content
                }
            
                $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
                file_put_contents($tempImagePath, $imageContent);
                $tempFiles[] = $tempImagePath;
                
                $imageInfo = @getimagesize($tempImagePath);
                if ($imageInfo === false) {
                    unlink($tempImagePath); // Remove the invalid image
                    continue;
                }

            $imageWidth = $imageInfo[0];
            $imageHeight = $imageInfo[1];
            $slideWidth = 960; // Default slide width (in points)
            $slideHeight = 720; // Default slide height (in points)
    
    
    // Scale the image to fit within the slide while maintaining aspect ratio
$scaleFactor = min($slideWidth / $imageWidth, $slideHeight / $imageHeight);
$newWidth = (int) round($imageWidth * $scaleFactor);
$newHeight = (int) round($imageHeight * $scaleFactor);
$offsetX = (int) round(($slideWidth - $newWidth) / 2);
$offsetY = (int) round(($slideHeight - $newHeight) / 2);

$currentSlide = $presentation->createSlide();
$shape = $currentSlide->createDrawingShape();
$shape->setName('Image ' . $index)
    ->setDescription('Slide ' . ($index + 1))
    ->setPath($tempImagePath)
    ->setWidth($newWidth)  // Now explicitly an integer
    ->setHeight($newHeight)  // Now explicitly an integer
    ->setOffsetX($offsetX)  // Now explicitly an integer
    ->setOffsetY($offsetY); // Now explicitly an integer

    $shape->getShadow()->setVisible(true)->setDirection(45)->setDistance(10);
    }

    $filePath = $directoryPath . $slidesName . '.pptx';

    // Ensure the directory exists
    if (!is_dir(dirname($filePath))) {
        mkdir(dirname($filePath), 0777, true); // Create the directory if it doesn't exist
    }

    // Create a writer instance and save the presentation
    $oWriterPPTX = new PhpOffice\PhpPresentation\Writer\PowerPoint2007($presentation);
    $oWriterPPTX->save($filePath);

    // Delete temp files
    foreach ($tempFiles as $tempFile) {
        if (file_exists($tempFile)) {
            unlink($tempFile);
        }
    }

            echo json_encode(['success' => true, 'message' => 'PPT generated successfully.', 'filename' => $slidesName . '.pptx']);
            exit;
        } else {
            echo json_encode(['success' => false, 'message' => 'Invalid output type specified.']);
        }
    } else {
        echo json_encode(['success' => false, 'message' => 'Invalid request method.']);
    }
} catch (Exception $e) {
    echo json_encode(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()]);
}
?>